rm(list=ls())
##############################
#### gauss hermit intergration
##############################

int=function(f1){

z=c(0.2453407083009, 0.7374737285454, 1.2340762153953, 1.7385377121166, 2.2549740020893, 
2.7888060584281, 3.3478545673832, 3.9447640401156, 4.6036824495507, 5.3874808900112, 
-0.2453407083009,-0.7374737285454,-1.2340762153953,-1.7385377121166,-2.2549740020893,
-2.7888060584281,-3.3478545673832,-3.9447640401156,-4.6036824495507,-5.3874808900112)
a=c(4.622436696006e-1,2.866755053628e-1,1.090172060200e-1,2.481052088746e-2,3.243773342238e-3,
2.283386360163e-4,7.802556478532e-6,1.086069370769e-7,4.399340992273e-10,2.229393645534e-13,     
4.622436696006e-1,2.866755053628e-1,1.090172060200e-1,2.481052088746e-2,3.243773342238e-3,
2.283386360163e-4,7.802556478532e-6,1.086069370769e-7,4.399340992273e-10,2.229393645534e-13)

ans=sum((f1(z))*exp(z^2/2)*a/sqrt(2*pi))

return(ans)

}

#########
### W_ij, ith subject and jth outcome
#########

WW=function(x, data, yhat, param, i, j){

my=dim(data$yy)[2]
mx=dim(data$xx)[2]
mz=dim(data$zz)[2]

beta=param$beta
gamma=param$gamma
alpha=param$alpha

cc= sum(beta[(1+(j-1)*mx):(j*mx)]*data$xx[i,])+alpha[j]*(sum(gamma*data$zz[i,])+x)
#cc=alpha[j]*(sum(gamma*data$zz[i,])+x)
ww=yhat[i]-cc

return(ww)

}

########################
# discrete W-ij strat
########################
yhat1=function(yhat){

### if yhat(j) is discrete ##
y1=sort(unique(yhat))
n=length(y1)
m1=match(yhat, y1)
y2=c(-99999, y1)[1:n]

return(y2[m1])

}

#############
### int F(x)
###############

intF=function(data, yhat, param){

n=dim(data$yy)[1]
mx=dim(data$xx)[2]

beta=param$beta
gamma=param$gamma
alpha=param$alpha

y2=yhat1(yhat[,2])

fi=rep(NA, n)
fb1=rep(NA, n)
fb2=rep(NA, n)
fa1=rep(NA, n)
fa2=rep(NA, n)

for(i in 1:n){

w1=function(x){
WW(x, data, yhat[,1], param, i, 1)
}

w2a=function(x){
WW(x, data, y2, param, i, 2)
}

w2=function(x){
WW(x, data, yhat[,2], param, i, 2)
}


ans=function(x){
dnorm(w1(x))*(pnorm(w2(x))-pnorm(w2a(x)))
}

fi[i]=int(ans)

ans=function(x){
dnorm(w1(x))*(pnorm(w2(x))-pnorm(w2a(x)))*w1(x)
}
fb1[i]=int(ans)

ans=function(x){
dnorm(w1(x))*(dnorm(w2(x))-dnorm(w2a(x)))
}

fb2[i]=int(ans)

ans=function(x){
dnorm(w1(x))*(pnorm(w2(x))-pnorm(w2a(x)))*w1(x)*x
}
fa1[i]=int(ans)

ans=function(x){
dnorm(w1(x))*(dnorm(w2(x))-dnorm(w2a(x)))*x
}
fa2[i]=int(ans)

##w1f=function(x){dnorm(w1(x))}

##w2f=function(x){pnorm(w2(x))-pnorm(w2a(x))}

##cc1=sum(beta[(1:mx)]*t(data$xx[i,]))+alpha[1]*(sum(gamma*t(data$zz[i,])))

##ww1=yhat[i,1]-cc1
##cc2=sum(beta[(mx+1):(2*mx)]*t(data$xx[i,]))+alpha[2]*(sum(gamma*t(data$zz[i,])))

##ww2=yhat[i,2]-cc2
##ww3=y2[i]-cc2


###print(c(i, fi[i], fb1[i], fb2[i], fa1[i], fa2[i]))

}
return(list(fi=fi, fb1=fb1, fb2=fb2, fa1=fa1, fa2=fa2))

}


##################
#### calculate deruivatives and information matrix
####################

derivative.f=function(data, param, ff){


n=dim(data$yy)[1]
vb11=vb12=vb13=vb14=vb15=vb16=vb21=vb22=vb23=vb24=vb25=vb26=va1=va2=vg1=rep(0,n)
### beta
for(i in 1:n){

if((ff$fi[i]==0)&(abs(ff$fb1[i])<1e-5))
{vb11[i]=data$xx[i,1]*(sign(ff$fb1[i])==0)+data$xx[i,1]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
 vb12[i]=data$xx[i,2]*(sign(ff$fb1[i])==0)+data$xx[i,2]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
 vb13[i]=data$xx[i,3]*(sign(ff$fb1[i])==0)+data$xx[i,3]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
 vb14[i]=data$xx[i,4]*(sign(ff$fb1[i])==0)+data$xx[i,4]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
 vb15[i]=data$xx[i,5]*(sign(ff$fb1[i])==0)+data$xx[i,5]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
 vb16[i]=data$xx[i,6]*(sign(ff$fb1[i])==0)+data$xx[i,6]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
} else
{
vb11[i]=ff$fb1[i]*data$xx[i,1]/ff$fi[i]
vb12[i]=ff$fb1[i]*data$xx[i,2]/ff$fi[i]
vb13[i]=ff$fb1[i]*data$xx[i,3]/ff$fi[i]
vb14[i]=ff$fb1[i]*data$xx[i,4]/ff$fi[i]
vb15[i]=ff$fb1[i]*data$xx[i,5]/ff$fi[i]
vb16[i]=ff$fb1[i]*data$xx[i,6]/ff$fi[i]
}
if(((ff$fi[i]==0)&(abs(ff$fb2[i])<1e-5))|((abs(ff$fi[i])<1e-5)&(ff$fb2[i]==0)))
{vb21[i]=-data$xx[i,1]*(sign(ff$fb2[i])==0)-data$xx[i,1]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
 vb22[i]=-data$xx[i,2]*(sign(ff$fb2[i])==0)-data$xx[i,2]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
 vb23[i]=-data$xx[i,3]*(sign(ff$fb2[i])==0)-data$xx[i,3]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
 vb24[i]=-data$xx[i,4]*(sign(ff$fb2[i])==0)-data$xx[i,4]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
 vb25[i]=-data$xx[i,5]*(sign(ff$fb2[i])==0)-data$xx[i,5]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
 vb26[i]=-data$xx[i,6]*(sign(ff$fb2[i])==0)-data$xx[i,6]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
} else
{vb21[i]= - ff$fb2[i]*data$xx[i,1]/ff$fi[i]
 vb22[i]= -(ff$fb2[i]*data$xx[i,2]/ff$fi[i])
 vb23[i]= -(ff$fb2[i]*data$xx[i,3]/ff$fi[i])
 vb24[i]= -(ff$fb2[i]*data$xx[i,4]/ff$fi[i])
 vb25[i]= -(ff$fb2[i]*data$xx[i,5]/ff$fi[i])
 vb26[i]= -(ff$fb2[i]*data$xx[i,6]/ff$fi[i])
}


### alpha
if((ff$fi[i]==0)&(abs(ff$fa1[i])<1e-5)&(abs(ff$fb1[i])<1e-5))
{
va1[i]=(sign(ff$fa1[i])==0)+(sign(ff$fa1[i])!=0)*sign(ff$fa1[i])+
       sum(data$zz[i,]*param$gamma)*(sign(ff$fb1[i])==0)+sum(data$zz[i,]*param$gamma)*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
} else
{va1[i]=ff$fa1[i]/ff$fi[i] + ff$fb1[i]*sum(data$zz[i,]*param$gamma)/ff$fi[i]}

if(((ff$fi[i]==0)&(abs(ff$fa2[i])<1e-5)&(abs(ff$fb2[i])<1e-5))|((ff$fi[i]<1e-5)&(ff$fa2[i]==0)&(ff$fb2[i]==0)))
{va2[i]=-(sign(ff$fa2[i])==0)-(sign(ff$fa2[i])!=0)*sign(ff$fa2[i])-
       sum(data$zz[i,]*param$gamma)*(sign(ff$fb2[i])==0)-sum(data$zz[i,]*param$gamma)*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])

} else
{
va2[i]=-(ff$fa2[i]/ff$fi[i]+ff$fb2[i]*sum(data$zz[i,]*param$gamma)/ff$fi[i])
}

## gamma
if(((ff$fi[i]==0)&(abs(ff$fb1[i])<1e-5)&(abs(ff$fb2[i])<1e-5))|((ff$fi[i]<1e-5)&(abs(ff$fb1[i])<1e-5)&(ff$fb2[i]==0)))
{vg1[i]=data$zz[i,]*param$alpha[1]*(sign(ff$fb1[i])==0)+
        data$zz[i,]*param$alpha[1]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])-
        data$zz[i,]*param$alpha[2]*(sign(ff$fb2[i])==0)-
        data$zz[i,]*param$alpha[2]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
} else
vg1[i]=(ff$fb1[i]*data$zz[i,]*param$alpha[1]/ff$fi[i]- ff$fb2[i]*data$zz[i,]*param$alpha[2]/ff$fi[i])

}

b11=sum(vb11)
b12=sum(vb12)
b13=sum(vb13)
b14=sum(vb14)
b15=sum(vb15)
b16=sum(vb16)
b21=sum(vb21)
b22=sum(vb22)
b23=sum(vb23)
b24=sum(vb24)
b25=sum(vb25)
b26=sum(vb26)
a1=sum(va1)
a2=sum(va2)
g1=sum(vg1)

### information

ib11b11=sum(vb11*vb11)
ib11b12=sum(vb11*vb12)
ib11b13=sum(vb11*vb13)
ib11b14=sum(vb11*vb14)
ib11b15=sum(vb11*vb15)
ib11b16=sum(vb11*vb16)
ib11b21=sum(vb11*vb21)
ib11b22=sum(vb11*vb22)
ib11b23=sum(vb11*vb23)
ib11b24=sum(vb11*vb24)
ib11b25=sum(vb11*vb25)
ib11b26=sum(vb11*vb26)
ib11a1=sum(vb11*va1)
ib11a2=sum(vb11*va2)
ib11g1=sum(vb11*vg1)

ib12b12=sum(vb12*vb12)
ib12b13=sum(vb12*vb13)
ib12b14=sum(vb12*vb14)
ib12b15=sum(vb12*vb15)
ib12b16=sum(vb12*vb16)
ib12b21=sum(vb12*vb21)
ib12b22=sum(vb12*vb22)
ib12b23=sum(vb12*vb23)
ib12b24=sum(vb12*vb24)
ib12b25=sum(vb12*vb25)
ib12b26=sum(vb12*vb26)
ib12a1=sum(vb12*va1)
ib12a2=sum(vb12*va2)
ib12g1=sum(vb12*vg1)

ib13b13=sum(vb13*vb13)
ib13b14=sum(vb13*vb14)
ib13b15=sum(vb13*vb15)
ib13b16=sum(vb13*vb16)
ib13b21=sum(vb13*vb21)
ib13b22=sum(vb13*vb22)
ib13b23=sum(vb13*vb23)
ib13b24=sum(vb13*vb24)
ib13b25=sum(vb13*vb25)
ib13b26=sum(vb13*vb26)
ib13a1=sum(vb13*va1)
ib13a2=sum(vb13*va2)
ib13g1=sum(vb13*vg1)

ib14b14=sum(vb14*vb14)
ib14b15=sum(vb14*vb15)
ib14b16=sum(vb14*vb16)
ib14b21=sum(vb14*vb21)
ib14b22=sum(vb14*vb22)
ib14b23=sum(vb14*vb23)
ib14b24=sum(vb14*vb24)
ib14b25=sum(vb14*vb25)
ib14b26=sum(vb14*vb26)
ib14a1=sum(vb14*va1)
ib14a2=sum(vb14*va2)
ib14g1=sum(vb14*vg1)

ib15b15=sum(vb15*vb15)
ib15b16=sum(vb15*vb16)
ib15b21=sum(vb15*vb21)
ib15b22=sum(vb15*vb22)
ib15b23=sum(vb15*vb23)
ib15b24=sum(vb15*vb24)
ib15b25=sum(vb15*vb25)
ib15b26=sum(vb15*vb26)
ib15a1=sum(vb15*va1)
ib15a2=sum(vb15*va2)
ib15g1=sum(vb15*vg1)

ib16b16=sum(vb16*vb16)
ib16b21=sum(vb16*vb21)
ib16b22=sum(vb16*vb22)
ib16b23=sum(vb16*vb23)
ib16b24=sum(vb16*vb24)
ib16b25=sum(vb16*vb25)
ib16b26=sum(vb16*vb26)
ib16a1=sum(vb16*va1)
ib16a2=sum(vb16*va2)
ib16g1=sum(vb16*vg1)


ib21b21=sum(vb21*vb21)
ib21b22=sum(vb21*vb22)
ib21b23=sum(vb21*vb23)
ib21b24=sum(vb21*vb24)
ib21b25=sum(vb21*vb25)
ib21b26=sum(vb21*vb26)
ib21a1=sum(vb21*va1)
ib21a2=sum(vb21*va2)
ib21g1=sum(vb21*vg1)

ib22b22=sum(vb22*vb22)
ib22b23=sum(vb22*vb23)
ib22b24=sum(vb22*vb24)
ib22b25=sum(vb22*vb25)
ib22b26=sum(vb22*vb26)
ib22a1=sum(vb22*va1)
ib22a2=sum(vb22*va2)
ib22g1=sum(vb22*vg1)

ib23b23=sum(vb23*vb23)
ib23b24=sum(vb23*vb24)
ib23b25=sum(vb23*vb25)
ib23b26=sum(vb23*vb26)
ib23a1=sum(vb23*va1)
ib23a2=sum(vb23*va2)
ib23g1=sum(vb23*vg1)

ib24b24=sum(vb24*vb24)
ib24b25=sum(vb24*vb25)
ib24b26=sum(vb24*vb26)
ib24a1=sum(vb24*va1)
ib24a2=sum(vb24*va2)
ib24g1=sum(vb24*vg1)

ib25b25=sum(vb25*vb25)
ib25b26=sum(vb25*vb26)
ib25a1=sum(vb25*va1)
ib25a2=sum(vb25*va2)
ib25g1=sum(vb25*vg1)

ib26b26=sum(vb26*vb26)
ib26a1=sum(vb26*va1)
ib26a2=sum(vb26*va2)
ib26g1=sum(vb26*vg1)


ia1a1=sum(va1*va1)
ia1a2=sum(va1*va2)
ia1g1=sum(va1*vg1)

ia2a2=sum(va2*va2)
ia2g1=sum(va2*vg1)

ig1g1=sum(vg1*vg1)

infm=matrix(c(ib11b11, ib11b12, ib11b13, ib11b14, ib11b15, ib11b16, ib11b21, ib11b22, ib11b23, ib11b24, ib11b25, ib11b26, ib11a1, ib11a2, ib11g1, 
		  ib11b12, ib12b12, ib12b13, ib12b14, ib12b15, ib12b16, ib12b21, ib12b22, ib12b23, ib12b24, ib12b25, ib12b26, ib12a1, ib12a2, ib12g1,
              ib11b13, ib12b13, ib13b13, ib13b14, ib13b15, ib13b16, ib13b21, ib13b22, ib13b23, ib13b24, ib13b25, ib13b26, ib13a1, ib13a2, ib13g1,
              ib11b14, ib12b14, ib13b14, ib14b14, ib14b15, ib14b16, ib14b21, ib14b22, ib14b23, ib14b24, ib14b25, ib14b26, ib14a1, ib14a2, ib14g1,
              ib11b15, ib12b15, ib13b15, ib14b15, ib15b15, ib15b16, ib15b21, ib15b22, ib15b23, ib15b24, ib15b25, ib15b26, ib15a1, ib15a2, ib15g1,
              ib11b16, ib12b16, ib13b16, ib14b16, ib15b16, ib16b16, ib16b21, ib16b22, ib16b23, ib16b24, ib16b25, ib16b26, ib16a1, ib16a2, ib16g1,
              ib11b21, ib12b21, ib13b21, ib14b21, ib15b21, ib16b21, ib21b21, ib21b22, ib21b23, ib21b24, ib21b25, ib21b26, ib21a1, ib21a2, ib21g1,
              ib11b22, ib12b22, ib13b22, ib14b22, ib15b22, ib16b22, ib21b22, ib22b22, ib22b23, ib22b24, ib22b25, ib22b26, ib22a1, ib22a2, ib22g1,
              ib11b23, ib12b23, ib13b23, ib14b23, ib15b23, ib16b23, ib21b23, ib22b23, ib23b23, ib23b24, ib23b25, ib23b26, ib23a1, ib23a2, ib23g1,
              ib11b24, ib12b24, ib13b24, ib14b24, ib15b24, ib16b24, ib21b24, ib22b24, ib23b24, ib24b24, ib24b25, ib24b26, ib24a1, ib24a2, ib24g1,
              ib11b25, ib12b25, ib13b25, ib14b25, ib15b25, ib16b25, ib21b25, ib22b25, ib23b25, ib24b25, ib25b25, ib25b26, ib25a1, ib25a2, ib25g1,
              ib11b26, ib12b26, ib13b26, ib14b26, ib15b26, ib16b26, ib21b26, ib22b26, ib23b26, ib24b26, ib25b26, ib26b26, ib26a1, ib26a2, ib26g1,
              ib11a1,  ib12a1,  ib13a1,  ib14a1,  ib15a1,  ib16a1,  ib21a1,  ib22a1,  ib23a1,  ib24a1,  ib25a1,  ib26a1,  ia1a1,  ia1a2,  ia1g1,
              ib11a2,  ib12a2,  ib13a2,  ib14a2,  ib15a2,  ib16a2,  ib21a2,  ib22a2,  ib23a2,  ib24a2,  ib25a2,  ib26a2,  ia1a2,  ia2a2,  ia2g1,
		  ib11g1,  ib12g1,  ib13g1,  ib14g1,  ib15g1,  ib16g1,  ib21g1,  ib22g1,  ib23g1,  ib24g1,  ib25g1,  ib26g1,  ia1g1,  ia2g1,  ig1g1), nrow=15)

return(list(deriv=c(b11, b12, b13, b14, b15, b16, b21, b22, b23, b24, b25, b26, a1, a2, g1)/n, infm=infm/n))

}

#############
### parameter estimation with fisher scoring method 
#############

estimate.param=function(data, yhat, param, delta){

d=delta+1
param1=param
i=0
v0=c(param1$beta, param1$alpha, param1$gamma)
v1=v0+1
while(d>delta){

d1=sum((v0-v1)^2)
ff=intF(data, yhat, param1)

#print(paste('loglik =', sum(log(ff$fi))))
deriv=derivative.f(data, param1, ff)
v0=c(param1$beta, param1$alpha, param1$gamma)

if(!is.na(sum(deriv$infm))){

qrinfo=qr(deriv$infm, tol=1e-9)
if(qrinfo$rank <15){
print("singular")
v1=rep(NA, 15)
} else {
inv=qr.coef(qrinfo, diag(rep(1,15)))
v1=v0+c(inv%*%(deriv$deriv))
}

} else v1=rep(NA, 15)

param1$beta=v1[1:12]
param1$alpha=v1[13:14]
param1$gamma=v1[15]

if(is.na(v1[1])) 
{
print(paste('##infor###',sum(deriv$infm)))
break
}
d2=sum((v0-v1)^2)

#print('v1')
# print(v1)
#print('v0')
#print(v0)
print(paste('d=', c(d1,d2)))
i=i+1
if(d< delta | i> 49 | abs(d1-d2)<0.0001 ) 
{
print(i)
print(paste('d=', c(d1,d2)))
break
}

}

return(param1)

}

##############################################
### newton raphon to solve 3.4 for each unique y
##############################################

newton1=function(x0, cc, ss, mm, delta){

d1=delta+1
s1=(x0-cc)/ss

while(d1 >delta){

xx=x0+(mm-sum(pnorm(s1)))/sum(dnorm(s1)/ss)
##print(sum(dnorm(s1)/ss))

x0=xx
s1=(x0-cc)/ss
##print(paste('x0=',x0, '  s1=',sum(dnorm(s1))))

d1=abs(mm-sum(pnorm(s1)))
if(is.na(d1)){ break
}
}
##print(paste('dd=',d1))

return(x0)

}
#########################################
### estimate transformation given parameters
###############################################

estimate.yhat=function(data, param, delta){

n=dim(data$yy)[1]
my=dim(data$yy)[2]
mx=dim(data$xx)[2]


yhat=matrix(NA, nrow=n, ncol=my)

beta=param$beta
gamma=param$gamma
alpha=param$alpha

## estimate of transformation
for( j in 1:my){

cc=(beta[(1+(j-1)*mx):(j*mx)]%*%t(data$xx)+alpha[j]*gamma%*%t(data$zz))
#cc=(alpha[j]*gamma%*%t(data$zz))
ss=sqrt(alpha[j]^2 +1)

## unique y
uu=sort(unique(data$yy[,j]))
nu=length(uu)

y2=rep(NA, nu)
#if(j==1) xs=min(log(data$yy[,j]))
#else xs=1
#xs=sum(beta[(1+(j-1)*mx):(j*mx)])/2+alpha[j]*gamma/2
xs=mean(data$xx%*%beta[(1+(j-1)*mx):(j*mx)])+alpha[j]*mean(gamma%*%t(data$zz))
#xs=alpha[j]*mean(gamma%*%t(data$zz))
print(xs)

for(i in 1:(nu-1)){
mm=sum(data$yy[,j]<=uu[i])
###print(paste('i=',i,' j=',j))
if(i==1) y2[i]=newton1(xs, cc, ss, mm, delta)
else y2[i]=newton1(y2[i-1], cc, ss, mm, delta)

if(is.na(y2[i])) y2[i]=newton1(xs, cc, ss, mm, delta)

if(is.na(y2[i])) 
{
print(paste("y2",i))
break

}
}

if(j==1) y2[nu]=y2[nu-1]
else 
{
y2[nu]=99999
#y2[1]=-99999
}
m1=match(data$yy[,j], uu)
yhat[,j]=y2[m1]
#yhat[,1]=data$yy[,1]
}
return(yhat)

}
############## ####### ############################# ####### ####
## analysis: iteration between trasformation and parameter estimation
### ########### ########## ########### ######### ######### ######

analysis=function(data, param, delta){

### data dimension
ydim=dim(data$yy)
xdim=dim(data$xx)
zdim=dim(data$zz)

##estimate hhat 

param.old=param

dd=delta+1
i=0
while(dd > delta){

print(paste('###', i, '###'))

yhat=estimate.yhat(data,  param.old, delta)
#yhat[,1]=data$yy[,1]
if(is.na(sum(yhat[,1])) | is.na(sum(yhat[,2]))) {
param.old$beta=rep(NA,12)
param.old$alpha=rep(NA,2)
param.old$gamma=NA
break
}
print('## yhat ##')
#### estimate other parameters

param.new=estimate.param(data, yhat, param.old, delta)

v0=c(param.old$beta, param.old$alpha, param.old$gamma)
#v0=c(param.old$alpha, param.old$gamma)
v1=c(param.new$beta, param.new$alpha, param.new$gamma)
#v1=c(param.new$alpha, param.new$gamma)
#print('v0')
#print(v0)
#print('v1')
#print(v1)

i=i+1
dd=sum((v0-v1)^2)
print(paste('dd=', dd))
param.old=param.new
if(i>150 | is.na(v1[1])|abs(dd)>1000) {
param.old$beta=rep(NA,12)
param.old$alpha=rep(NA,2)
param.old$gamma=NA
break
}

}

return(list(param=param.old, yhat=yhat))

}
a=read.table(file="ninds_updated_all.txt",header=TRUE)
a1=a[,c(13,314,22,18,26,27,23,40,188)]

a1=matrix(as.numeric(as.matrix(a1,nrow(a1),ncol(a1))),nrow(a1),ncol(a1))
nrow(a1)
a2=a1[!is.na(apply(a1,1,sum)),]
nrow(a2)
y=a2[,c(9:8)]

a3=a2

###sex###
a3[a2[,4]==2,4]=0
####edema###
a3[a2[,5]==2,5]=0
####mass effect###
a3[a2[,6]==2,6]=0

x=matrix(a3[,c(2:7)],nrow(a2),6)
z=matrix(a3[,c(1)],nrow(a2),1)
z[z==2]=0

#summary(lm(a3[,9]~x+z))

data1=list(yy=y,xx=x,zz=z)
delta=0.001

param=list(beta=c(0, 0.24, 0.57, -1.41, 10.73, 0.03,
                  0, 0.04, 0.24, -0.02, 1.14,  0.01), gamma=1, alpha=c(-2.54, -0.57))
#param=list(beta=c(0.047, 0.116, 0.830, 0.485, 0.886, 0.055,
 #                 0.006, 0.029, 0.183, 0.093, 0.919, 0.006), gamma=0.236, alpha=c(-2.006, -1.247))


out1 =analysis(data1, param, delta)
print(out1$param)
mm1=sort(data1$yy[,1])
mm2=match(mm1,data1$yy[,1])
if(!is.na(sum(out1$yhat[,1]))){
plot(data1$yy[mm2,1],out1$yhat[mm2,1],lty=3,col="blue",type="l")
}
if(!is.na(sum(out1$yhat[,1]))){
outy21=sort(unique(out1$yhat[,2]))
print(outy21)
}


write.table(out1$yhat[,1],"ey_2_f_6.txt")
rs=out1$yhat[,1]-x%*%out1$param$beta[1:6]-out1$param$alpha[1]*z%*%out1$param$gamma
print(c('1',c(mean(rs),var(rs),out1$param$alpha[1]^2+1)))

