rm(list=ls())

##############################
#### gauss hermit intergration
##############################

int=function(f1){

z=c(0.2453407083009, 0.7374737285454, 1.2340762153953, 1.7385377121166, 2.2549740020893, 
2.7888060584281, 3.3478545673832, 3.9447640401156, 4.6036824495507, 5.3874808900112, 
-0.2453407083009,-0.7374737285454,-1.2340762153953,-1.7385377121166,-2.2549740020893,
-2.7888060584281,-3.3478545673832,-3.9447640401156,-4.6036824495507,-5.3874808900112)
a=c(4.622436696006e-1,2.866755053628e-1,1.090172060200e-1,2.481052088746e-2,3.243773342238e-3,
2.283386360163e-4,7.802556478532e-6,1.086069370769e-7,4.399340992273e-10,2.229393645534e-13,     
4.622436696006e-1,2.866755053628e-1,1.090172060200e-1,2.481052088746e-2,3.243773342238e-3,
2.283386360163e-4,7.802556478532e-6,1.086069370769e-7,4.399340992273e-10,2.229393645534e-13)

ans=sum((f1(z))*exp(z^2/2)*a/sqrt(2*pi))

return(ans)

}

#########
### W_ij, ith subject and jth outcome
#########

WW=function(x, data, yhat, param, i, j){

my=dim(data$yy)[2]
mx=dim(data$xx)[2]
mz=dim(data$zz)[2]

beta=param$beta
gamma=param$gamma
alpha=param$alpha

cc= sum(beta[(1+(j-1)*mx):(j*mx)]*data$xx[i,])+alpha[j]*(sum(gamma*data$zz[i,])+x)

ww=yhat[i]-cc

return(ww)

}

########################
# discrete W-ij strat
########################
yhat1=function(yhat){

### if yhat(j) is discrete ##
y1=sort(unique(yhat))
n=length(y1)
m1=match(yhat, y1)
y2=c(-99999, y1)[1:n]

return(y2[m1])

}

#############
### int F(x)
###############

intF=function(data, yhat, param){

n=dim(data$yy)[1]
mx=dim(data$xx)[2]

beta=param$beta
gamma=param$gamma
alpha=param$alpha

y2=yhat1(yhat[,2])

fi=rep(NA, n)
fb1=rep(NA, n)
fb2=rep(NA, n)
fa1=rep(NA, n)
fa2=rep(NA, n)

for(i in 1:n){

w1=function(x){
WW(x, data, yhat[,1], param, i, 1)
}

w2a=function(x){
WW(x, data, y2, param, i, 2)
}

w2=function(x){
WW(x, data, yhat[,2], param, i, 2)
}


ans=function(x){
dnorm(w1(x))*(pnorm(w2(x))-pnorm(w2a(x)))
}

fi[i]=int(ans)

ans=function(x){
dnorm(w1(x))*(pnorm(w2(x))-pnorm(w2a(x)))*w1(x)
}
fb1[i]=int(ans)

ans=function(x){
dnorm(w1(x))*(dnorm(w2(x))-dnorm(w2a(x)))
}

fb2[i]=int(ans)

ans=function(x){
dnorm(w1(x))*(pnorm(w2(x))-pnorm(w2a(x)))*w1(x)*x
}
fa1[i]=int(ans)

ans=function(x){
dnorm(w1(x))*(dnorm(w2(x))-dnorm(w2a(x)))*x
}
fa2[i]=int(ans)

##w1f=function(x){dnorm(w1(x))}

##w2f=function(x){pnorm(w2(x))-pnorm(w2a(x))}

##cc1=sum(beta[(1:mx)]*t(data$xx[i,]))+alpha[1]*(sum(gamma*t(data$zz[i,])))

##ww1=yhat[i,1]-cc1
##cc2=sum(beta[(mx+1):(2*mx)]*t(data$xx[i,]))+alpha[2]*(sum(gamma*t(data$zz[i,])))

##ww2=yhat[i,2]-cc2
##ww3=y2[i]-cc2


###print(c(i, fi[i], fb1[i], fb2[i], fa1[i], fa2[i]))

}
return(list(fi=fi, fb1=fb1, fb2=fb2, fa1=fa1, fa2=fa2))

}

##################
#### calculate deruivatives and information matrix
####################

derivative.f=function(data, param, ff){


n=dim(data$yy)[1]
vb11=vb12=vb21=vb22=va1=va2=vg1=rep(0,n)
### beta
for(i in 1:n){

if((ff$fi[i]==0)&(abs(ff$fb1[i])<1e-5))
{vb11[i]=data$xx[i,1]*(sign(ff$fb1[i])==0)+data$xx[i,1]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
 vb12[i]=data$xx[i,2]*(sign(ff$fb1[i])==0)+data$xx[i,2]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
} else
{vb11[i]=ff$fb1[i]*data$xx[i,1]/ff$fi[i]
vb12[i]=ff$fb1[i]*data$xx[i,2]/ff$fi[i]
}
if(((ff$fi[i]==0)&(abs(ff$fb2[i])<1e-5))|((abs(ff$fi[i])<1e-5)&(ff$fb2[i]==0)))
{vb21[i]=-data$xx[i,1]*(sign(ff$fb2[i])==0)-data$xx[i,1]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
 vb22[i]=-data$xx[i,2]*(sign(ff$fb2[i])==0)-data$xx[i,2]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
} else
{vb21[i]= - ff$fb2[i]*data$xx[i,1]/ff$fi[i]
vb22[i]= -(ff$fb2[i]*data$xx[i,2]/ff$fi[i])
}


### alpha
if((ff$fi[i]==0)&(abs(ff$fa1[i])<1e-5)&(abs(ff$fb1[i])<1e-5))
{
va1[i]=(sign(ff$fa1[i])==0)+(sign(ff$fa1[i])!=0)*sign(ff$fa1[i])+
       data$zz[i]*param$gamma*(sign(ff$fb1[i])==0)+data$zz[i]*param$gamma*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])
} else
{va1[i]=ff$fa1[i]/ff$fi[i] + ff$fb1[i]*data$zz[i]*param$gamma/ff$fi[i]}

if(((ff$fi[i]==0)&(abs(ff$fa2[i])<1e-5)&(abs(ff$fb2[i])<1e-5))|((ff$fi[i]<1e-5)&(ff$fa2[i]==0)&(ff$fb2[i]==0)))
{va2[i]=-(sign(ff$fa2[i])==0)-(sign(ff$fa2[i])!=0)*sign(ff$fa2[i])-
       data$zz[i]*param$gamma*(sign(ff$fb2[i])==0)-data$zz[i]*param$gamma*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])

} else
{
va2[i]=-(ff$fa2[i]/ff$fi[i]+ff$fb2[i]*data$zz[i]*param$gamma/ff$fi[i])
}

## gamma
if(((ff$fi[i]==0)&(abs(ff$fb1[i])<1e-5)&(abs(ff$fb2[i])<1e-5))|((ff$fi[i]<1e-5)&(abs(ff$fb1[i])<1e-5)&(ff$fb2[i]==0)))
{vg1[i]=data$zz[i]*param$alpha[1]*(sign(ff$fb1[i])==0)+
        data$zz[i]*param$alpha[1]*(sign(ff$fb1[i])!=0)*sign(ff$fb1[i])-
        data$zz[i]*param$alpha[2]*(sign(ff$fb2[i])==0)-
        data$zz[i]*param$alpha[2]*(sign(ff$fb2[i])!=0)*sign(ff$fb2[i])
} else
vg1[i]=(ff$fb1[i]*data$zz[i]*param$alpha[1]/ff$fi[i]- ff$fb2[i]*data$zz[i]*param$alpha[2]/ff$fi[i])

}

b11=sum(vb11)
b12=sum(vb12)
b21=sum(vb21)
b22=sum(vb22)
a1=sum(va1)
a2=sum(va2)
g1=sum(vg1)

### information

ib11b11=sum(vb11*vb11)
ib11b12=sum(vb11*vb12)
ib11b21=sum(vb11*vb21)
ib11b22=sum(vb11*vb22)
ib11a1=sum(vb11*va1)
ib11a2=sum(vb11*va2)
ib11g1=sum(vb11*vg1)

ib12b12=sum(vb12*vb12)
ib12b21=sum(vb12*vb21)
ib12b22=sum(vb12*vb22)
ib12a1=sum(vb12*va1)
ib12a2=sum(vb12*va2)
ib12g1=sum(vb12*vg1)

ib21b21=sum(vb21*vb21)
ib21b22=sum(vb21*vb22)
ib21a1=sum(vb21*va1)
ib21a2=sum(vb21*va2)
ib21g1=sum(vb21*vg1)

ib22b22=sum(vb22*vb22)
ib22a1=sum(vb22*va1)
ib22a2=sum(vb22*va2)
ib22g1=sum(vb22*vg1)

ia1a1=sum(va1*va1)
ia1a2=sum(va1*va2)
ia1g1=sum(va1*vg1)

ia2a2=sum(va2*va2)
ia2g1=sum(va2*vg1)

ig1g1=sum(vg1*vg1)

infm=matrix(c(ib11b11, ib11b12, ib11b21, ib11b22, ib11a1, ib11a2, ib11g1,
		  ib11b12, ib12b12, ib12b21, ib12b22, ib12a1, ib12a2, ib12g1,
		  ib11b21, ib12b21, ib21b21, ib21b22, ib21a1, ib21a2, ib21g1,
	        ib11b22, ib12b22, ib21b22, ib22b22, ib22a1, ib22a2, ib22g1,
		  ib11a1,  ib12a1,  ib21a1,  ib22a1,  ia1a1,  ia1a2,  ia1g1,
		  ib11a2,  ib12a2,  ib21a2,  ib22a2,  ia1a2,  ia2a2,  ia2g1,
		  ib11g1,  ib12g1,  ib21g1,  ib22g1,  ia1g1,  ia2g1,  ig1g1), nrow=7)

return(list(deriv=c(b11, b12, b21, b22, a1, a2, g1)/n, infm=infm/n))

}


#############
### parameter estimation with fisher scoring method 
#############

estimate.param=function(data, yhat, param, delta){

d=delta+1
param1=param
i=0
while(d>delta){

ff=intF(data, yhat, param1)

## print(paste('loglik =', sum(log(ff$fi))))
deriv=derivative.f(data, param1, ff)
v0=c(param1$beta, param1$alpha, param1$gamma)
if(!is.na(sum(deriv$infm))){

qrinfo=qr(deriv$infm, tol=1e-9)
if(qrinfo$rank <7){
v1=rep(NA, 7)
} else {
inv=qr.coef(qrinfo, diag(rep(1,7)))
v1=v0+c(inv%*%(deriv$deriv))
}

} else v1=rep(NA, 7)

param1$beta=v1[1:4]
param1$alpha=v1[5:6]
param1$gamma=v1[7]

if(is.na(v1[1])) break

d=sum((v0-v1)^2)
#print('v1')
 #print(v1)
#print('v0')
#print(v0)
 #print(paste('d=', d))
i=i+1
if(d< delta | i> 20 )
{
 print(paste('d=', d))
 break
}
}

return(param1)

}


##############################################
### newton raphon to solve 3.4 for each unique y
##############################################

newton1=function(x0, cc, ss, mm, delta){

d1=delta+1
s1=(x0-cc)/ss

while(d1 >delta){

xx=x0+(mm-sum(pnorm(s1)))/sum(dnorm(s1)/ss)
##print(sum(dnorm(s1)/ss))

x0=xx
s1=(x0-cc)/ss
##print(paste('x0=',x0, '  s1=',sum(dnorm(s1))))

d1=abs(mm-sum(pnorm(s1)))
if(is.na(d1)){ break
}
}
##print(paste('dd=',d1))

return(x0)

}

#########################################
### estimate transformation given parameters
###############################################

estimate.yhat=function(data, param, delta){

n=dim(data$yy)[1]
my=dim(data$yy)[2]
mx=dim(data$xx)[2]

yhat=matrix(NA, nrow=n, ncol=my)

beta=param$beta
gamma=param$gamma
alpha=param$alpha

yhat[,1]=log(data$yy[,1])
y2=uu=sort(unique(data$yy[,2]))
nu=length(y2)
m1=match(data$yy[,2],uu)
y2[nu]=99999
yhat[,2]=y2[m1]
return(yhat)

}

############## ####### ############################# ####### ####
## analysis: iteration between trasformation and parameter estimation
### ########### ########## ########### ######### ######### ######

analysis=function(data, param, delta){

### data dimension
ydim=dim(data$yy)
xdim=dim(data$xx)
zdim=dim(data$zz)

##estimate hhat 

param.old=param

dd=delta+1
i=0
while(dd > delta){

print(paste('###', i, '###'))

yhat=estimate.yhat(data,  param.old, delta)
if(is.na(sum(yhat[,1])) | is.na(sum(yhat[,2]))) {
param.old$beta=rep(NA,4)
param.old$alpha=rep(NA,2)
param.old$gamma=NA
break
}
print('## yhat ##')
#### estimate other parameters

param.new=estimate.param(data, yhat, param.old, delta)

v0=c(param.old$beta, param.old$alpha, param.old$gamma)
v1=c(param.new$beta, param.new$alpha, param.new$gamma)
#print('v0')
#print(v0)
#print('v1')
#print(v1)

i=i+1
dd=sum((v0-v1)^2)
print(paste('dd=', dd))
param.old=param.new
if(i>150 | is.na(v1[1])) {
param.old$beta=rep(NA,4)
param.old$alpha=rep(NA,2)
param.old$gamma=NA
break
}

}

return(list(param=param.old, yhat=yhat))

}

###################

#### generating data
#####################

data1.gen=function(n, dimm, param, cutoff){

my=dimm$my
mx=dimm$mx
mz=dimm$mz

beta=param$beta
alpha=param$alpha
gamma=param$gamma

yy=matrix(NA, nrow=n, ncol=my)
## trandom effects
yr=matrix(NA, nrow=n, ncol=my)
xx=matrix(NA, nrow=n, ncol=mx)
zz=matrix(NA, nrow=n, ncol=mz)

### generate x
for(k in 1:mx){
xx[,k]=runif(n)
}

### generate z
for(k in 1:mz){
##zz[,k]=rbinom(n, 1, 0.3)
zz[,k]=runif(n)

}

### generate random effects

zr=apply(gamma*t(zz),2,sum) + (rgamma(n,100,1)-100)/10

### generate y1

t1=apply(beta[1:mx]*t(xx),2, sum)+alpha[1]*zr+(rgamma(n,100,1)-100)/10

### transformation
yy[,1]=exp(t1)
## yy[,1]=t1
### generate y2

t2=apply(beta[(mx+1):(2*mx)]*t(xx),2,sum)+alpha[2]*zr+(rgamma(n,100,1)-100)/10
### transformation
cc=length(cutoff)
yy[,2]=1
for(k in 1:cc)
yy[t2>cutoff[k] ,2]=k+1

return(list(yy=yy, xx=xx, zz=zz))

}

#### simulation ####
sim1=function(n, dimm, param, delta=0.0001, cutoff, nsim,ly){

out=matrix(NA, nrow=nsim, ncol=7)
outy1=matrix(NA,nrow=nsim,ncol=n)
outy3=outy2=matrix(NA,nrow=nsim,ncol=5)
#vv=matrix(NA, nrow=nsim, ncol=7)


for(i in 1:nsim){
set.seed(i)
 print(paste('### new sim ##########', i))
data2=data1 = data1.gen(n, dimm, param, cutoff)
data2$yy=ly

###out1 =analysis2(data1, param, delta)

out1 =analysis(data1, param, delta)

out[i,]=c(out1$param$beta, out1$param$alpha, out1$param$gamma)

}

return(list(out=out))

}


dimm=list(my=2, mx=2, mz=1)
param=list(beta=c(1.5,1.5,2,2), gamma=3, alpha=c(0.5, 0.5))
n=300
delta=0.001
cutoff=c(1,2,3,4)

set.seed(1)
data=data1.gen(n, dimm, param, cutoff)
ly=data$yy

out1=sim1(n, dimm, param, delta, cutoff, nsim=500,ly)



write.table(out1$out,"theta_e2_ini2_t.txt")





